@@ -8,7 +8,7 @@ from werkzeug.wrappers import BaseResponse
 from flask import jsonify
 
 from .utils import (
-    deepupdate, get_appcontext, prepare_response,
+    deepupdate, remove_none, get_appcontext, prepare_response,
     unpack_tuple_response, set_status_and_headers_in_response
 )
 from .spec import DEFAULT_RESPONSE_CONTENT_TYPE
@@ -18,15 +18,15 @@ class ResponseMixin:
     """Extend Blueprint to add response handling"""
 
     def response(
-            self, schema=None, *, code=200, description=None,
+            self, status_code, schema=None, *, description=None,
             example=None, examples=None, headers=None
     ):
         """Decorator generating an endpoint response
 
+        :param int|str|HTTPStatus status_code: HTTP status code.
+            Used if none is returned from the view function.
         :param schema: :class:`Schema <marshmallow.Schema>` class or instance.
             If not None, will be used to serialize response data.
-        :param int|str|HTTPStatus code: HTTP status code (default: 200).
-            Used if none is returned from the view function.
         :param str description: Description of the response (default: None).
         :param dict example: Example of response message.
         :param list examples: Examples of response message.
@@ -38,7 +38,7 @@ class ResponseMixin:
         a ``string``.
 
         If the decorated function returns a ``Response`` object, the ``schema``
-        and ``code`` parameters are only used to document the resource.
+        and ``status_code`` parameters are only used to document the resource.
 
         The `example` and `examples` parameters are mutually exclusive. The
         latter should only be used with OpenAPI 3.
@@ -52,21 +52,16 @@ class ResponseMixin:
             schema = schema()
 
         # Document response (schema, description,...) in the API doc
-        resp_doc = {}
         doc_schema = self._make_doc_response_schema(schema)
-        if doc_schema is not None:
-            resp_doc['schema'] = doc_schema
-        if description is not None:
-            resp_doc['description'] = description
-        else:
-            resp_doc['description'] = http.HTTPStatus(int(code)).phrase
-        if example is not None:
-            resp_doc['example'] = example
-        if examples is not None:
-            resp_doc['examples'] = examples
-        if headers is not None:
-            resp_doc['headers'] = headers
-        doc = {'responses': {code: resp_doc}}
+        if description is None:
+            description = http.HTTPStatus(int(status_code)).phrase
+        resp_doc = remove_none({
+            "schema": doc_schema,
+            "description": description,
+            "example": example,
+            "examples": examples,
+            "headers": headers,
+        })
 
         def decorator(func):
 
@@ -74,13 +69,13 @@ class ResponseMixin:
             def wrapper(*args, **kwargs):
 
                 # Execute decorated function
-                result_raw, status, headers = unpack_tuple_response(
+                result_raw, r_status_code, r_headers = unpack_tuple_response(
                     func(*args, **kwargs))
 
                 # If return value is a werkzeug BaseResponse, return it
                 if isinstance(result_raw, BaseResponse):
                     set_status_and_headers_in_response(
-                        result_raw, status, headers)
+                        result_raw, r_status_code, r_headers)
                     return result_raw
 
                 # Dump result with schema if specified
@@ -96,19 +91,76 @@ class ResponseMixin:
 
                 # Build response
                 resp = jsonify(self._prepare_response_content(result_dump))
-                set_status_and_headers_in_response(resp, status, headers)
-                if status is None:
-                    resp.status_code = code
+                set_status_and_headers_in_response(
+                    resp, r_status_code, r_headers
+                )
+                if r_status_code is None:
+                    resp.status_code = status_code
 
                 return resp
 
             # Store doc in wrapper function
             # The deepcopy avoids modifying the wrapped function doc
             wrapper._apidoc = deepcopy(getattr(wrapper, '_apidoc', {}))
-            wrapper._apidoc['response'] = doc
+            wrapper._apidoc.setdefault(
+                'response', {}
+            ).setdefault('responses', {})[status_code] = resp_doc
             # Indicate which code is the success status code
             # Helps other decorators documenting success response
-            wrapper._apidoc['success_status_code'] = code
+            wrapper._apidoc['success_status_code'] = status_code
+
+            return wrapper
+
+        return decorator
+
+    def alt_response(
+            self, status_code, schema_or_ref, *, description=None,
+            example=None, examples=None, headers=None
+    ):
+        """Decorator documenting an alternative response
+
+        :param int|str|HTTPStatus status_code: HTTP status code.
+        :param schema_or_ref: Either a :class:`Schema <marshmallow.Schema>`
+            class or instance or a string error reference.
+            When passing a reference, arguments below are ignored.
+        :param str description: Description of the response (default: None).
+        :param dict example: Example of response message.
+        :param list examples: Examples of response message.
+        :param dict headers: Headers returned by the response.
+        """
+        # If a ref is passed
+        if isinstance(schema_or_ref, str):
+            resp_doc = schema_or_ref
+        # If a schema is passed
+        else:
+            schema = schema_or_ref
+            if isinstance(schema, type):
+                schema = schema()
+
+            # Document response (schema, description,...) in the API doc
+            doc_schema = self._make_doc_response_schema(schema)
+            if description is None:
+                description = http.HTTPStatus(int(status_code)).phrase
+            resp_doc = remove_none({
+                "schema": doc_schema,
+                "description": description,
+                "example": example,
+                "examples": examples,
+                "headers": headers,
+            })
+
+        def decorator(func):
+
+            @wraps(func)
+            def wrapper(*args, **kwargs):
+                return func(*args, **kwargs)
+
+            # Store doc in wrapper function
+            # The deepcopy avoids modifying the wrapped function doc
+            wrapper._apidoc = deepcopy(getattr(wrapper, '_apidoc', {}))
+            wrapper._apidoc.setdefault(
+                'response', {}
+            ).setdefault('responses', {})[status_code] = resp_doc
 
             return wrapper
 
@@ -116,7 +168,7 @@ class ResponseMixin:
 
     @staticmethod
     def _make_doc_response_schema(schema):
-        """Override this to modify schema in docs
+        """Override this to modify response schema in docs
 
         This can be used to document a wrapping structure.
 
