@@ -461,7 +461,7 @@ class TestBlueprint:
 
         @blp.route('/', methods=('GET', 'POST', ))
         @blp.arguments(schemas.DocSchema)
-        @blp.response(schemas.DocSchema)
+        @blp.response(200, schemas.DocSchema)
         def func(document, query_args):
             pass
 
@@ -502,12 +502,12 @@ class TestBlueprint:
         blp = Blueprint('test', 'test', url_prefix='/test')
 
         @blp.route('/schema_many_false')
-        @blp.response(schemas.DocSchema(many=False))
+        @blp.response(200, schemas.DocSchema(many=False))
         def many_false():
             pass
 
         @blp.route('/schema_many_true')
-        @blp.response(schemas.DocSchema(many=True))
+        @blp.response(200, schemas.DocSchema(many=True))
         def many_true():
             pass
 
@@ -540,12 +540,12 @@ class TestBlueprint:
         blp = Blueprint('test', 'test', url_prefix='/test')
 
         @blp.route('/route_1')
-        @blp.response(code=204)
+        @blp.response(204)
         def func_1():
             pass
 
         @blp.route('/route_2')
-        @blp.response(code=204, description='Test')
+        @blp.response(204, description='Test')
         def func_2():
             pass
 
@@ -568,7 +568,7 @@ class TestBlueprint:
         example = {'name': 'One'}
 
         @blp.route('/')
-        @blp.response(example=example)
+        @blp.response(200, example=example)
         def func():
             pass
 
@@ -595,7 +595,7 @@ class TestBlueprint:
         }
 
         @blp.route('/')
-        @blp.response(examples=examples)
+        @blp.response(200, examples=examples)
         def func():
             pass
 
@@ -612,7 +612,7 @@ class TestBlueprint:
         headers = {'X-Header': {'description': 'Custom header'}}
 
         @blp.route('/')
-        @blp.response(headers=headers)
+        @blp.response(200, headers=headers)
         def func():
             pass
 
@@ -652,6 +652,172 @@ class TestBlueprint:
                 api.spec, 'response', error_response
             )
 
+    @pytest.mark.parametrize('openapi_version', ['2.0', '3.0.2'])
+    def test_blueprint_alt_response_schema(
+            self, app, openapi_version, schemas
+    ):
+        """Check alternate response schema is correctly documented"""
+        app.config['OPENAPI_VERSION'] = openapi_version
+        api = Api(app)
+        blp = Blueprint('test', 'test', url_prefix='/test')
+
+        example = {'error_id': 'E1', 'text': 'client error'}
+        examples = {
+            'example 1': {'error_id': 'E1', 'text': 'client error 1'},
+            'example 2': {'error_id': 'E2', 'text': 'client error 2'},
+        }
+        headers = {'X-Custom-Header': 'Header value'}
+
+        @blp.route('/')
+        @blp.alt_response(400, schemas.ClientErrorSchema)
+        def func():
+            pass
+
+        @blp.route('/description')
+        @blp.alt_response(
+            400, schemas.ClientErrorSchema, description='Client error'
+        )
+        def func_with_description():
+            pass
+
+        @blp.route('/example')
+        @blp.alt_response(400, schemas.ClientErrorSchema, example=example)
+        def func_with_example():
+            pass
+
+        if openapi_version == '3.0.2':
+            @blp.route('/examples')
+            @blp.alt_response(
+                400, schemas.ClientErrorSchema, examples=examples
+            )
+            def func_with_examples():
+                pass
+
+        @blp.route('/headers')
+        @blp.alt_response(400, schemas.ClientErrorSchema, headers=headers)
+        def func_with_headers():
+            pass
+
+        api.register_blueprint(blp)
+
+        paths = api.spec.to_dict()['paths']
+
+        schema_ref = build_ref(api.spec, 'schema', 'ClientError')
+
+        response = paths['/test/']['get']['responses']['400']
+        if openapi_version == '2.0':
+            assert response['schema'] == schema_ref
+        else:
+            assert (
+                response['content']['application/json']['schema'] ==
+                schema_ref
+            )
+        assert response['description'] == http.HTTPStatus(400).phrase
+
+        response = paths['/test/description']['get']['responses']['400']
+        assert response['description'] == 'Client error'
+
+        response = paths['/test/example']['get']['responses']['400']
+        if openapi_version == '2.0':
+            assert response['examples']['application/json'] == example
+        else:
+            assert (
+                response['content']['application/json']['example'] == example
+            )
+
+        if openapi_version == '3.0.2':
+            response = paths['/test/examples']['get']['responses']['400']
+            assert (
+                response['content']['application/json']['examples'] == examples
+            )
+
+        response = paths['/test/headers']['get']['responses']['400']
+        assert response['headers'] == headers
+
+    @pytest.mark.parametrize('openapi_version', ['2.0', '3.0.2'])
+    def test_blueprint_alt_response_ref(self, app, openapi_version):
+        """Check alternate response passed as reference"""
+        app.config['OPENAPI_VERSION'] = openapi_version
+        api = Api(app)
+        api.spec.components.response('ClientErrorResponse')
+
+        blp = Blueprint('test', 'test', url_prefix='/test')
+
+        @blp.route('/')
+        @blp.alt_response(400, "ClientErrorResponse")
+        def func():
+            pass
+
+        api.register_blueprint(blp)
+
+        paths = api.spec.to_dict()['paths']
+
+        response_ref = build_ref(api.spec, 'response', 'ClientErrorResponse')
+
+        assert paths['/test/']['get']['responses']['400'] == response_ref
+
+    @pytest.mark.parametrize('openapi_version', ['2.0', '3.0.2'])
+    def test_blueprint_multiple_alt_response(
+            self, app, openapi_version, schemas
+    ):
+        """Check multiple nested calls to alt_response"""
+        app.config['OPENAPI_VERSION'] = openapi_version
+        api = Api(app)
+        blp = Blueprint('test', 'test', url_prefix='/test')
+
+        @blp.route('/')
+        @blp.alt_response(400, schemas.ClientErrorSchema)
+        @blp.alt_response(404, 'NotFoundErrorResponse')
+        def func():
+            pass
+
+        api.register_blueprint(blp)
+
+        paths = api.spec.to_dict()['paths']
+
+        schema_ref = build_ref(api.spec, 'schema', 'ClientError')
+        response_ref = build_ref(api.spec, 'response', 'NotFoundErrorResponse')
+
+        response = paths['/test/']['get']['responses']['400']
+        if openapi_version == '2.0':
+            assert response['schema'] == schema_ref
+        else:
+            assert (
+                response['content']['application/json']['schema'] ==
+                schema_ref
+            )
+
+        assert paths['/test/']['get']['responses']['404'] == response_ref
+
+    @pytest.mark.parametrize('openapi_version', ['2.0', '3.0.2'])
+    def test_blueprint_alt_response_wrapper(
+            self, app, schemas, openapi_version
+    ):
+        """Check alt_response passes response transparently"""
+        app.config['OPENAPI_VERSION'] = openapi_version
+        api = Api(app)
+        api.spec.components.response('ClientErrorResponse')
+
+        blp = Blueprint('test', 'test', url_prefix='/test')
+        client = app.test_client()
+
+        @blp.route('/')
+        @blp.response(200, schemas.DocSchema)
+        @blp.alt_response(400, "ClientErrorResponse")
+        def func():
+            return {'item_id': 12}
+
+        api.register_blueprint(blp)
+
+        paths = api.spec.to_dict()['paths']
+
+        response_ref = build_ref(api.spec, 'response', 'ClientErrorResponse')
+
+        assert paths['/test/']['get']['responses']['400'] == response_ref
+
+        resp = client.get('test/')
+        assert resp.json == {'item_id': 12}
+
     @pytest.mark.parametrize('openapi_version', ('2.0', '3.0.2'))
     def test_blueprint_pagination(self, app, schemas, openapi_version):
         app.config['OPENAPI_VERSION'] = openapi_version
@@ -802,7 +968,7 @@ class TestBlueprint:
             @blp.doc(**{'requestBody': doc_example})
             @blp.doc(**{'responses': {200: doc_example}})
             @blp.arguments(ItemSchema)
-            @blp.response(ItemSchema)
+            @blp.response(200, ItemSchema)
             def get(self):
                 pass
 
@@ -835,7 +1001,7 @@ class TestBlueprint:
             # (Default is 200 expressed as int.)
             @blp.doc(**{'responses': {status_code: doc_desc}})
             @blp.arguments(ItemSchema)
-            @blp.response(ItemSchema, code=status_code)
+            @blp.response(status_code, ItemSchema)
             def get(self):
                 pass
 
@@ -964,42 +1130,42 @@ class TestBlueprint:
         client = app.test_client()
 
         @blp.route('/response')
-        @blp.response()
+        @blp.response(200)
         def func_response():
             return {}
 
         @blp.route('/response_code_int')
-        @blp.response()
+        @blp.response(200)
         def func_response_code_int():
             return {}, 201
 
         @blp.route('/response_code_str')
-        @blp.response()
+        @blp.response(200)
         def func_response_code_str():
             return {}, '201 CREATED'
 
         @blp.route('/response_headers')
-        @blp.response()
+        @blp.response(200)
         def func_response_headers():
             return {}, {'X-header': 'test'}
 
         @blp.route('/response_code_int_headers')
-        @blp.response()
+        @blp.response(200)
         def func_response_code_int_headers():
             return {}, 201, {'X-header': 'test'}
 
         @blp.route('/response_code_str_headers')
-        @blp.response()
+        @blp.response(200)
         def func_response_code_str_headers():
             return {}, '201 CREATED', {'X-header': 'test'}
 
         @blp.route('/response_wrong_tuple')
-        @blp.response()
+        @blp.response(200)
         def func_response_wrong_tuple():
             return {}, 201, {'X-header': 'test'}, 'extra'
 
         @blp.route('/response_tuple_subclass')
-        @blp.response()
+        @blp.response(200)
         def func_response_tuple_subclass():
             class MyTuple(tuple):
                 pass
@@ -1044,37 +1210,37 @@ class TestBlueprint:
         client = app.test_client()
 
         @blp.route('/response')
-        @blp.response()
+        @blp.response(200)
         @blp.paginate(Page)
         def func_response():
             return [1, 2]
 
         @blp.route('/response_code')
-        @blp.response()
+        @blp.response(200)
         @blp.paginate(Page)
         def func_response_code():
             return [1, 2], 201
 
         @blp.route('/response_headers')
-        @blp.response()
+        @blp.response(200)
         @blp.paginate(Page)
         def func_response_headers():
             return [1, 2], {'X-header': 'test'}
 
         @blp.route('/response_code_headers')
-        @blp.response()
+        @blp.response(200)
         @blp.paginate(Page)
         def func_response_code_headers():
             return [1, 2], 201, {'X-header': 'test'}
 
         @blp.route('/response_wrong_tuple')
-        @blp.response()
+        @blp.response(200)
         @blp.paginate(Page)
         def func_response_wrong_tuple():
             return [1, 2], 201, {'X-header': 'test'}, 'extra'
 
         @blp.route('/response_tuple_subclass')
-        @blp.response()
+        @blp.response(200)
         @blp.paginate(Page)
         def func_response_tuple_subclass():
             class MyTuple(tuple):
@@ -1110,7 +1276,7 @@ class TestBlueprint:
 
         @blp.route('/response')
         # Schema is ignored when response object is returned
-        @blp.response(schemas.DocSchema, code=200)
+        @blp.response(200, schemas.DocSchema)
         def func_response():
             return flask.jsonify({"test": "test"}), 201, {'X-header': 'test'}
 
